@@ -61,7 +61,11 @@ class RangeRequestHandler(SimpleHTTPRequestHandler):
         if 'Range' not in self.headers:
             self.range = None
             return SimpleHTTPRequestHandler.send_head(self)
-        self.range = parse_byte_range(self.headers['Range'])
+        try:
+            self.range = parse_byte_range(self.headers['Range'])
+        except ValueError as e:
+            self.send_error(400, 'Invalid byte range')
+            return None
         first, last = self.range
 
         # Mirroring SimpleHTTPServer.py here
@@ -73,14 +77,23 @@ class RangeRequestHandler(SimpleHTTPRequestHandler):
         except IOError:
             self.send_error(404, 'File not found')
             return None
+
+        fs = os.fstat(f.fileno())
+        file_len = fs[6]
+        if first >= file_len:
+            self.send_error(416, 'Requested Range Not Satisfiable')
+            return None
+
         self.send_response(206)
         self.send_header('Content-type', ctype)
         self.send_header('Accept-Ranges', 'bytes')
 
-        fs = os.fstat(f.fileno())
-        response_length = last - first + 1 if last else fs[6] - first
+        if last is None or last >= file_len:
+            last = file_len - 1
+        response_length = last - first + 1
 
-        self.send_header('Content-Range', 'bytes %s-%s/%s' % (first, last, fs[6]))
+        self.send_header('Content-Range',
+                         'bytes %s-%s/%s' % (first, last, file_len))
         self.send_header('Content-Length', str(response_length))
         self.send_header('Last-Modified', self.date_time_string(fs.st_mtime))
         self.end_headers()
